# API Docs

## Sourcemapper

### keyvalue *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#source-mappers">Source Mapper</a>)*

<p style="word-wrap: break-word">Key-Value Map to Event input mapper. Transports which accepts key value maps can utilize this extension to convert the incoming key value pairs to Siddhi event. Users can either send predefined keys where conversion will happen without any configs or can use custom keys to map from message</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
@source(..., @map(type="keyvalue", fail.on.missing.attribute="<BOOL>")
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">fail.on.missing.attribute</td>
        <td style="vertical-align: top; word-wrap: break-word">This can either have value true or false. By default it will be true. This attribute allows user to handle unknown attributes. By default if a key is missing in a message Stream Processor will drop that message. However setting this property to false will prompt Stream Processor to send an event with null value to Siddhi where user can handle it accordingly</td>
        <td style="vertical-align: top">true</td>
        <td style="vertical-align: top">BOOL</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@source(type='inMemory', topic='stock', @map(type='keyvalue'))
define stream FooStream (symbol string, price float, volume long);

```
<p style="word-wrap: break-word">Above configuration will do a default key value input mapping. Expected input will be a Map as follows,symbol: 'WSO2'price: 55.6fvolume: 100</p>

<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
@source(type='inMemory', topic='stock', @map(type='keyvalue', fail.on.missing.attribute='true', @attributes(symbol = 's', price = 'p', volume = 'v')))define stream FooStream (symbol string, price float, volume long); 
```
<p style="word-wrap: break-word">Above configuration will do a custom key value input mapping. Expected inputwill be a Map as follows,s: 'WSO2'p: 55.6v: 100</p>

## Sinkmapper

### keyvalue *(<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#sink-mappers">Sink Mapper</a>)*

<p style="word-wrap: break-word">Event to Key-Value Map output mapper. Transports which publish messages can utilize this extension to convert the Siddhi event to Key-Value Map message. Users can either user predefined keys and values or use custom keys and values</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
@sink(..., @map(type="keyvalue")
```

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@sink(type='inMemory', topic='stock', @map(type='keyvalue'))
define stream FooStream (symbol string, price float, volume long);

```
<p style="word-wrap: break-word">Above configuration will perform a default Key-Value output mapping.Expected output will be a Map as follows,symbol:'WSO2'price : 55.6fvolume: 100L</p>

<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
@sink(type='inMemory', topic='stock', @map(type='keyvalue', @payload(a='symbol',b='price',c='volume')))
define stream FooStream (symbol string, price float, volume long);

```
<p style="word-wrap: break-word">Above configuration will perform a custom Key-Value output mapping where valuesvalues are passed as objectsExpected output will be a Map as follows,a:'WSO2'b : 55.6fc: 100L</p>

<span id="example-3" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 3</span>
```
@sink(type='inMemory', topic='stock', @map(type='keyvalue', @payload(a='{{symbol}} is here',b='`price`',c='volume')))
define stream FooStream (symbol string, price float, volume long);

```
<p style="word-wrap: break-word">Above configuration will perform a custom Key-Value output mapping where valuesof a,b are Strings and c is objectExpected output will be a Map as follows,a:'WSO2 is here'b : 'price'c: 100L</p>

